@@ -3,10 +3,12 @@ from enum import Enum
 
 import dolfin
 import pulse
-import ufl
 from mpi4py import MPI
 
 from . import utils
+from .newton_solver import MechanicsNewtonSolver
+from .newton_solver import MechanicsNewtonSolver_ODE
+
 
 logger = utils.getLogger(__name__)
 
@@ -16,174 +18,125 @@ class BoundaryConditions(str, Enum):
     rigid = "rigid"
 
 
-class LandModel(pulse.ActiveModel):
-    def __init__(
-        self,
-        XS,
-        XW,
-        dt,
-        lmbda,
-        Zetas,
-        Zetaw,
-        parameters,
-        function_space,
-        f0=None,
-        s0=None,
-        n0=None,
-        eta=0,
-        **kwargs,
-    ):
-        super().__init__(f0=f0, s0=s0, n0=n0)
-        self._eta = eta
-        self.function_space = function_space
-
-        self.XS = XS
-        self.XS_prev = dolfin.Function(function_space)
-        self.XW = XW
-        self.XW_prev = dolfin.Function(function_space)
-        self._parameters = parameters
-        self.dt = dt
-
-        self.Zetas = Zetas
-        self.Zetaw = Zetaw
-
-        self.Zetas_prev = dolfin.Function(self.Zetas.function_space())
-        self.Zetas_prev_prev = dolfin.Function(self.Zetas.function_space())
-
-        self.Zetaw_prev = dolfin.Function(self.Zetaw.function_space())
-        self.Zetaw_prev_prev = dolfin.Function(self.Zetaw.function_space())
-
-        self.Ta_current = dolfin.Function(function_space, name="Ta")
-        self.lmbda_prev = dolfin.Function(function_space)
-        self.lmbda_current = dolfin.Function(function_space)
-        self.lmbda_current = lmbda
-        self.update_prev()
-
-    def update_prev(self):
-        self.XS_prev.vector()[:] = self.XS.vector()
-        self.XW_prev.vector()[:] = self.XW.vector()
-
-        self.Zetas_prev_prev.vector()[:] = self.Zetas_prev.vector()
-        self.Zetaw_prev_prev.vector()[:] = self.Zetaw_prev.vector()
-        self.Zetas_prev.vector()[:] = self.Zetas.vector()
-        self.Zetaw_prev.vector()[:] = self.Zetaw.vector()
-        self.lmbda_prev.vector()[:] = self.lmbda_current.vector()
-
-    def lmbda(self, F):
-        f = F * self.f0
-        return dolfin.sqrt(f ** 2)
-
-    def dLambda(self, F):
-        dLambda = (self.lmbda(F) - self.lmbda_prev) / self.dt
-        return dLambda
-
-    def _solve_ode(self, F):
-
-        phi = self._parameters["phi"]
-        Tot_A = self._parameters["Tot_A"]
-        kuw = self._parameters["kuw"]
-        kws = self._parameters["kws"]
-        rs = self._parameters["rs"]
-        rw = self._parameters["rw"]
-
-        scale_popu_kuw = self._parameters["scale_popu_kuw"]
-        scale_popu_kws = self._parameters["scale_popu_kws"]
-        scale_popu_rw = self._parameters["scale_popu_rw"]
-        scale_popu_rs = self._parameters["scale_popu_rs"]
-
-        Aw = (
-            Tot_A
-            * rs
-            * scale_popu_rs
-            / (rs * scale_popu_rs + rw * scale_popu_rw * (1 - (rs * scale_popu_rs)))
-        )
-        As = Aw
-
-        cw = (
-            kuw
-            * scale_popu_kuw
-            * phi
-            * (1 - (rw * scale_popu_rw))
-            / (rw * scale_popu_rw)
-        )
-        cs = (
-            kws
-            * scale_popu_kws
-            * phi
-            * rw
-            * scale_popu_rw
-            * (1 - (rs * scale_popu_rs))
-            / (rs * scale_popu_rs)
-        )
-
-        # dZetas = self.dLambda * As - self.Zetas * cs
-        # dZetaw = self.dLambda * Aw - self.Zetaw * cw
-
-        # Lets use Backward Euler scheme with a few fixed point iterations
-        Zetas = self.Zetas
-        Zetaw = self.Zetaw
-
-        for _ in range(10):
-            Zetas = self.Zetas_prev + self.dt * (self.dLambda(F) * As - Zetas * cs)
-            Zetaw = self.Zetaw_prev + self.dt * (self.dLambda(F) * Aw - Zetaw * cw)
-
-        return Zetas, Zetaw
-
-    def Ta(self, F):
-
-        Zetas, Zetaw = self._solve_ode(F)
-        Tref = self._parameters["Tref"]
-        rs = self._parameters["rs"]
-        scale_popu_Tref = self._parameters["scale_popu_Tref"]
-        scale_popu_rs = self._parameters["scale_popu_rs"]
-        Beta0 = self._parameters["Beta0"]
-        lmbda = self.lmbda(F)
-
-        _min = ufl.min_value
-        _max = ufl.max_value
-        if isinstance(lmbda, (int, float)):
-            _min = min
-            _max = max
-        lmbda = _min(1.2, lmbda)
-        h_lambda_prima = 1 + Beta0 * (lmbda + _min(lmbda, 0.87) - 1.87)
-        h_lambda = _max(0, h_lambda_prima)
-
-        Ta = (
-            h_lambda
-            * (Tref * scale_popu_Tref / (rs * scale_popu_rs))
-            * (self.XS * (Zetas + 1) + self.XW * Zetaw)
-        )
-
-        # Assign the current value of Ta so that we can retrive them for postprocessing
-        self.Ta_current.assign(dolfin.project(Ta, self.function_space))
-        # Assign these in order to update the EM coupling
-        self.lmbda_current.assign(dolfin.project(lmbda, self.function_space))
-        self.Zetas.assign(dolfin.project(Zetas, self.function_space))
-        self.Zetaw.assign(dolfin.project(Zetaw, self.function_space))
-
-        return Ta
-
-    def Wactive(self, F, diff=0):
-        """Active stress energy"""
-        C = F.T * F
-
-        if diff == 0:
-            return pulse.material.active_model.Wactive_transversally(
-                Ta=self.Ta(F),
-                C=C,
-                f0=self.f0,
-                eta=self.eta,
-            )
-        return self.Ta(F)
-
-
-class MechanicsProblem(pulse.MechanicsProblem):
-    boundary_condition = BoundaryConditions.dirichlet
+class ContinuationBasedMechanicsProblem(pulse.MechanicsProblem):
+    def __init__(self, *args, **kwargs):
+        self._use_custom_newton_solver = kwargs.pop("use_custom_newton_solver", False)
+        super().__init__(*args, **kwargs)
+        self.old_states = []
+        self.old_controls = []
 
     def solve(self):
         self._init_forms()
         return super().solve()
 
+    def solve_for_control(self, control, tol=1e-5):
+        """Solve with a continuation step for
+        better initial guess for the Newton solver
+        """
+        if len(self.old_controls) >= 2:
+            # Find a better initial guess for the solver
+            c0, c1 = self.old_controls
+            s0, s1 = self.old_states
+
+            denominator = dolfin.assemble((c1 - c0) ** 2 * dolfin.dx)
+            max_denom = self.geometry.mesh.mpi_comm().allreduce(denominator, op=MPI.MAX)
+
+            if max_denom > tol:
+                numerator = dolfin.assemble((control - c0) ** 2 * dolfin.dx)
+                delta = numerator / denominator
+                self.state.vector().zero()
+                self.state.vector().axpy(1.0 - delta, s0.vector())
+                self.state.vector().axpy(delta, s1.vector())
+
+                # Keep track of the newest state
+                self.old_controls = [c1]
+                self.old_states = [s1]
+            else:
+                # Keep track of the old state
+                self.old_controls = [c0]
+                self.old_states = [s0]
+
+        self.solve()
+
+        self.old_states.append(self.state.copy(deepcopy=True))
+        self.old_controls.append(control.copy(deepcopy=True))
+
+
+class MechanicsProblem(ContinuationBasedMechanicsProblem):
+    boundary_condition = BoundaryConditions.dirichlet
+
+    def _init_spaces(self):
+
+        mesh = self.geometry.mesh
+
+        P1 = dolfin.FiniteElement("Lagrange", mesh.ufl_cell(), 1)
+        P2 = dolfin.VectorElement("Lagrange", mesh.ufl_cell(), 2)
+
+        self.state_space = dolfin.FunctionSpace(
+            mesh,
+            dolfin.MixedElement([P2, P1]),
+        )
+        self._init_functions()
+
+    def _init_functions(self):
+        self.state = dolfin.Function(self.state_space, name="state")
+        self.state_test = dolfin.TestFunction(self.state_space)
+
+    def _init_forms(self):
+        u, p = dolfin.split(self.state)
+
+        # Some mechanical quantities
+        self._F = dolfin.variable(pulse.DeformationGradient(u))
+        self._J = pulse.Jacobian(self._F)
+        dx = self.geometry.dx
+
+        internal_energy = self.material.strain_energy(
+            self._F,
+        ) + self.material.compressibility(p, self._J)
+
+        self._virtual_work = dolfin.derivative(
+            internal_energy * dx,
+            self.state,
+            self.state_test,
+        )
+
+        self._set_dirichlet_bc()
+        self._jacobian = dolfin.derivative(
+            self._virtual_work,
+            self.state,
+            dolfin.TrialFunction(self.state_space),
+        )
+        self._init_solver()
+
+    def _init_solver(self):
+        self._problem = pulse.NonlinearProblem(
+            J=self._jacobian,
+            F=self._virtual_work,
+            bcs=self._dirichlet_bc,
+        )
+
+        if self._use_custom_newton_solver:
+            cls = MechanicsNewtonSolver_ODE
+        else:
+            cls = MechanicsNewtonSolver
+
+        self.solver = cls(
+            problem=self._problem,
+            state=self.state,
+            update_cb=self.material.active.update_prev,
+            parameters=self.solver_parameters,
+        )
+
+    def solve(self):
+        self._init_forms()
+        newton_iteration, newton_converged = self.solver.solve()
+        # DEBUGGING
+        getattr(self.solver, "check_overloads_called", None)
+        return newton_iteration, newton_converged
+
+    def update_lmbda_prev(self):
+        self.lmbda_prev.vector()[:] = self.lmbda.vector()
+
 
 class RigidMotionProblem(MechanicsProblem):
     boundary_condition = BoundaryConditions.rigid
@@ -196,30 +149,30 @@ class RigidMotionProblem(MechanicsProblem):
         P2 = dolfin.VectorElement("Lagrange", mesh.ufl_cell(), 2)
         P3 = dolfin.VectorElement("Real", mesh.ufl_cell(), 0, 6)
 
-        self.state_space = dolfin.FunctionSpace(mesh, dolfin.MixedElement([P1, P2, P3]))
+        self.state_space = dolfin.FunctionSpace(
+            mesh,
+            dolfin.MixedElement([P1, P2, P3]),
+        )
 
-        self.state = dolfin.Function(self.state_space, name="state")
-        self.state_test = dolfin.TestFunction(self.state_space)
+        self._init_functions()
 
     def _handle_bcs(self, bcs, bcs_parameters):
         self.bcs = pulse.BoundaryConditions()
+        self._dirichlet_bc = []
 
     def _init_forms(self):
         # Displacement and hydrostatic_pressure; 3rd space for rigid motion component
         p, u, r = dolfin.split(self.state)
-        q, v, w = dolfin.split(self.state_test)
+        q, v, z = dolfin.split(self.state_test)
 
         # Some mechanical quantities
-        F = dolfin.variable(pulse.DeformationGradient(u))
-        J = pulse.Jacobian(F)
+        self._F = dolfin.variable(pulse.DeformationGradient(u))
+        self._J = pulse.Jacobian(self._F)
         dx = self.geometry.dx
 
-        internal_energy = (
-            self.material.strain_energy(
-                F,
-            )
-            + self.material.compressibility(p, J)
-        )
+        internal_energy = self.material.strain_energy(
+            self._F,
+        ) + self.material.compressibility(p, self._J)
 
         self._virtual_work = dolfin.derivative(
             internal_energy * dx,
@@ -244,18 +197,6 @@ class RigidMotionProblem(MechanicsProblem):
         )
         self._init_solver()
 
-    def _init_solver(self):
-        self._problem = pulse.NonlinearProblem(
-            J=self._jacobian,
-            F=self._virtual_work,
-            bcs=[],
-        )
-        self.solver = pulse.NonlinearSolver(
-            self._problem,
-            self.state,
-            parameters=self.solver_parameters,
-        )
-
     def rigid_motion_term(mesh, u, r):
         position = dolfin.SpatialCoordinate(mesh)
         RM = [
@@ -272,7 +213,8 @@ class RigidMotionProblem(MechanicsProblem):
 
 def setup_microstructure(mesh):
     logger.debug("Set up microstructure")
-    V_f = dolfin.VectorFunctionSpace(mesh, "DG", 1)
+    # V_f = dolfin.VectorFunctionSpace(mesh, "DG", 1)
+    V_f = pulse.QuadratureSpace(mesh, degree=3, dim=3)
     f0 = dolfin.interpolate(
         dolfin.Expression(("1.0", "0.0", "0.0"), degree=1, cell=mesh.ufl_cell()),
         V_f,
@@ -291,7 +233,7 @@ def setup_microstructure(mesh):
 
 def float_to_constant(x: typing.Union[dolfin.Constant, float]) -> dolfin.Constant:
     """Convert float to a dolfin constant.
-    If value is allready a constant, do nothing.
+    If value is already a constant, do nothing.
 
     Parameters
     ----------
@@ -313,10 +255,11 @@ def setup_diriclet_bc(
     pre_stretch: typing.Optional[typing.Union[dolfin.Constant, float]] = None,
     traction: typing.Union[dolfin.Constant, float] = None,
     spring: typing.Union[dolfin.Constant, float] = None,
-    fix_right_plane: bool = True,
+    fix_right_plane: bool = False,
 ) -> typing.Tuple[pulse.BoundaryConditions, pulse.MarkerFunctions]:
-    """Completely fix the left side of the mesh (i.e the side with the
-    lowest x-values) and apply some boundary condition to the right side.
+    """Completely fix the left side of the mesh in the x-direction (i.e the side with the
+    lowest x-values), fix points at x=0 & y=0 in y-direction, fix points at x=0 & z=0 in
+    z-direction and apply some boundary condition to the right side.
 
 
     Parameters
@@ -361,6 +304,16 @@ def setup_diriclet_bc(
     # Define domain to apply dirichlet boundary conditions
     left = dolfin.CompiledSubDomain("near(x[0], 0) && on_boundary")
     right = dolfin.CompiledSubDomain("near(x[0], Lx) && on_boundary", Lx=Lx)
+    leftback = dolfin.CompiledSubDomain("near(x[0], 0) && near(x[2], 0)")
+    leftbottom = dolfin.CompiledSubDomain("near(x[0], 0) && near(x[1], 0)")
+    rightback = dolfin.CompiledSubDomain(
+        "near(x[0], Lx) && near(x[2], 0) && on_boundary",
+        Lx=Lx,
+    )
+    rightbottom = dolfin.CompiledSubDomain(
+        "near(x[0], Lx) && near(x[1], 0) && on_boundary",
+        Lx=Lx,
+    )
 
     boundary_markers = dolfin.MeshFunction("size_t", mesh, mesh.topology().dim() - 1)
     boundary_markers.set_all(0)
@@ -369,6 +322,10 @@ def setup_diriclet_bc(
     left.mark(boundary_markers, left_marker)
     right_marker = 2
     right.mark(boundary_markers, right_marker)
+    leftback_marker = 3
+    leftback.mark(boundary_markers, leftback_marker)
+    leftbottom_marker = 4
+    leftbottom.mark(boundary_markers, leftbottom_marker)
 
     marker_functions = pulse.MarkerFunctions(ffun=boundary_markers)
 
@@ -378,24 +335,43 @@ def setup_diriclet_bc(
         # BC with fixing left size
         bcs = [
             dolfin.DirichletBC(
-                W.sub(0),
-                dolfin.Constant((0.0, 0.0, 0.0)),
+                W.sub(0).sub(0),  # u_x
+                dolfin.Constant(0.0),
                 left,
             ),
+            dolfin.DirichletBC(
+                W.sub(0).sub(1),  # u_y
+                dolfin.Constant(0.0),
+                leftbottom,
+                method="pointwise",
+            ),
+            dolfin.DirichletBC(
+                W.sub(0).sub(2),  # u_z
+                dolfin.Constant(0.0),
+                leftback,
+                method="pointwise",
+            ),
         ]
 
         if fix_right_plane:
             bcs.extend(
                 [
                     dolfin.DirichletBC(
-                        W.sub(0).sub(1),  # Second component of u, i.e u_y
-                        dolfin.Constant(0.0),  # should be kept fixed
-                        right,  # in this region
+                        W.sub(0).sub(0),  # u_x
+                        dolfin.Constant(0.0),
+                        right,
                     ),
                     dolfin.DirichletBC(
-                        W.sub(0).sub(2),  # Third component of u, i.e u_z
-                        dolfin.Constant(0.0),  # should be kept fixed
-                        right,  # in this region
+                        W.sub(0).sub(1),  # u_y
+                        dolfin.Constant(0.0),
+                        rightbottom,
+                        method="pointwise",
+                    ),
+                    dolfin.DirichletBC(
+                        W.sub(0).sub(2),  # u_z
+                        dolfin.Constant(0.0),
+                        rightback,
+                        method="pointwise",
                     ),
                 ],
             )
