@@ -4,6 +4,7 @@ import warnings
 from pathlib import Path
 
 import dolfin
+import pulse
 from dolfin import FiniteElement  # noqa: F401
 from dolfin import MixedElement  # noqa: F401
 from dolfin import tetrahedron  # noqa: F401
@@ -110,8 +111,8 @@ def save_state(
     logger.debug("Save using dolfin.HDF5File")
     with dolfin.HDF5File(ep_mesh.mpi_comm(), path.as_posix(), "w") as h5file:
         h5file.write(mech_heart.material.active.lmbda_prev, "/em/lmbda_prev")
-        h5file.write(mech_heart.material.active.Zetas_prev_prev, "/em/Zetas_prev")
-        h5file.write(mech_heart.material.active.Zetaw_prev_prev, "/em/Zetaw_prev")
+        h5file.write(mech_heart.material.active.Zetas_prev, "/em/Zetas_prev")
+        h5file.write(mech_heart.material.active.Zetaw_prev, "/em/Zetaw_prev")
 
         h5file.write(ep_mesh, "/ep/mesh")
         h5file.write(solver.vs, "/ep/vs")
@@ -144,8 +145,6 @@ def load_state(
     disease_state="healthy",
 ):
     logger.debug(f"Load state from path {path}")
-    if drug_factors_file != "" or popu_factors_file != "":
-        logger.info("Load drug or population factors from file")
     path = Path(path)
     if not path.is_file():
         raise FileNotFoundError(f"File {path} does not exist")
@@ -156,7 +155,7 @@ def load_state(
         if "geometry_params" in h5file:
             geometry_params = h5_to_dict(h5file["geometry_params"])
         else:
-            # For backwards compatability
+            # For backwards compatibility
             warnings.warn(
                 (
                     f"Unable to find 'geometry_params' in result file {path}. "
@@ -192,7 +191,7 @@ def load_state(
     W = dolfin.FunctionSpace(mech_mesh, eval(mech_signature))
     mech_state = dolfin.Function(W)
 
-    V = dolfin.FunctionSpace(mech_mesh, "CG", 1)
+    V = pulse.QuadratureSpace(mech_mesh, degree=3, dim=1)
     lmbda_prev = dolfin.Function(V, name="lambda")
     Zetas_prev = dolfin.Function(V, name="Zetas")
     Zetaw_prev = dolfin.Function(V, name="Zetaw")
@@ -207,9 +206,6 @@ def load_state(
 
     coupling = em_model.EMCoupling(
         geometry=geo,
-        lmbda_mech=lmbda_prev,
-        Zetas_mech=Zetas_prev,
-        Zetaw_mech=Zetaw_prev,
     )
     solver = setup_models.setup_ep_solver(
         state_params["dt"],
@@ -225,11 +221,15 @@ def load_state(
 
     mech_heart = setup_models.setup_mechanics_solver(
         coupling=coupling,
-        dt=state_params["dt"],
         bnd_cond=bnd_cond_dict[state_params["bnd_cond"]],
         cell_params=solver.ode_solver._model.parameters(),
+        Zetas_prev=Zetas_prev,
+        Zetaw_prev=Zetaw_prev,
+        lmbda_prev=lmbda_prev,
+        state_prev=mech_state,
     )
-    mech_heart.state.assign(mech_state)
+
+    coupling.coupling_to_mechanics()
 
     return setup_models.EMState(
         coupling=coupling,
